package cn.com.ty.lift.enterprise.oa.controller;

import cn.com.ty.lift.common.model.CountPage;
import cn.com.ty.lift.common.verify.Ver;
import cn.com.ty.lift.common.verify.Verifier;
import cn.com.ty.lift.common.verify.Verify;
import cn.com.ty.lift.enterprise.oa.dto.AttendanceRequest;
import cn.com.ty.lift.enterprise.oa.dto.AttendanceResponse;
import cn.com.ty.lift.enterprise.oa.entity.Attendance;
import cn.com.ty.lift.enterprise.oa.entity.MaintenanceCompany;
import cn.com.ty.lift.enterprise.oa.service.AttendanceImgService;
import cn.com.ty.lift.enterprise.oa.service.AttendanceService;
import cn.com.ty.lift.enterprise.oa.service.MaintenanceCompanyService;
import cn.com.xwy.boot.web.dto.RestResponse;
import cn.hutool.core.io.IoUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.poi.excel.ExcelUtil;
import cn.hutool.poi.excel.ExcelWriter;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.time.LocalTime;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *      打卡签到记录 前端控制器
 * </p>
 *
 * @author wcz
 * @since 2019-12-04
 */
@Slf4j
@RestController
@AllArgsConstructor
@RequestMapping("/oa/attendance")
public class AttendanceController {

    private AttendanceService         attendanceService;
    private AttendanceImgService      attendanceImgService;
    private MaintenanceCompanyService maintenanceCompanyService;

    private static final Map<String, String> headerAlias = new HashMap<>();

    static {
        headerAlias.put("username","姓名");
        headerAlias.put("typeDesc","打卡类型");
        headerAlias.put("clockDate","日期");
        headerAlias.put("requiredTime","应打卡时间");
        headerAlias.put("clockTime","打卡时间");
        headerAlias.put("address","地点");
        headerAlias.put("remarks","备注");
        headerAlias.put("statusDesc","是否迟到/早退");
    }

    /**
     * 根据id查找
     * @param request AttendanceRequest
     * @return RestResponse
     */
    @PostMapping("findOne")
    @Verifier(fields = {"id"})
    public RestResponse findOne(@Ver @RequestBody AttendanceRequest request) {
        AttendanceResponse result = attendanceService.infoById(request);
        if(null != result){
            result.setImgs(attendanceImgService.listByAttendance(request.getId()));
        }
        return RestResponse.success(result);
    }

    /**
     * 根据条件查询
     * @param request AttendanceRequest
     * @return RestResponse
     */
    @PostMapping("list")
    @Verifier(fields = "mtCompanyId")
    public RestResponse list(@Ver @RequestBody AttendanceRequest request) {
//        LocalDate begin = request.getBegin();
//        LocalDate end = request.getEnd();
//        if(null != begin && null != end){
//            long until = end.until(begin, ChronoUnit.DAYS);
//            Verify.notTrue(until > ValuePool.QUERY_MAX_DAYS, "最多只允许查询间隔" + ValuePool.QUERY_MAX_DAYS + "天的记录");
//        }else if(null != begin && null == end){
//            request.setEnd(begin.plusDays(ValuePool.QUERY_MAX_DAYS));
//        }else if(null == begin && null != end){
//            request.setBegin(end.plusDays(0 - ValuePool.QUERY_MAX_DAYS));
//        }else{
//            request.setBegin(LocalDate.now().plusDays(0 - ValuePool.QUERY_MAX_DAYS));
//            request.setEnd(LocalDate.now());
//        }
        IPage<AttendanceResponse> pages = attendanceService.pageByCondition(request);
        long count = attendanceService.count();
        return RestResponse.success(CountPage.getCountPage(count, pages));
    }

    /**
     * 设置打卡时间
     * @param request AttendanceRequest
     * @return RestResponse
     */
    @PostMapping("setupTime")
    @Verifier(fields = {"mtCompanyId"})
    public RestResponse setupTime(@Ver @RequestBody AttendanceRequest request) {
        LocalTime topTime = request.getTopTime();
        LocalTime downTime = request.getDownTime();
        boolean top = ObjectUtil.isNotNull(topTime);
        boolean down = ObjectUtil.isNotNull(downTime);
        boolean am = top && down;
        //上下班时间成对设置
        Verify.notTrue(top ^ down, Verify.Attend.SetAmTime);
        if(am){
            Verify.notTrue(topTime.isAfter(downTime), Verify.Attend.InvalidAmTime);
        }

        LocalTime topTimePm = request.getTopTimePm();
        LocalTime downTimePm = request.getDownTimePm();
        boolean toppm = ObjectUtil.isNotNull(topTimePm);
        boolean downpm = ObjectUtil.isNotNull(downTimePm);
        boolean pm = toppm && downpm;
        Verify.notTrue(toppm ^ downpm, Verify.Attend.SetPmTime);
        if(pm){
            Verify.notTrue(topTimePm.isAfter(downTimePm), Verify.Attend.InvalidPmTime);
        }
        //上午下班 < 下午上班
        if(down && toppm){
            Verify.notTrue(downTime.isAfter(topTimePm),Verify.Attend.AmDownPmTop);
        }

        LocalTime topTimeOt = request.getTopTimeOt();
        LocalTime downTimeOt = request.getDownTimeOt();
        boolean topot = ObjectUtil.isNotNull(topTimeOt);
        boolean downot = ObjectUtil.isNotNull(downTimeOt);
        boolean ot = topot && downot;
        Verify.notTrue(topot ^ downot, Verify.Attend.SetOtTime);
        if(ot){
            Verify.notTrue(topTimeOt.isAfter(downTimeOt), Verify.Attend.InvalidOtTime);
        }
        // 下午下班 < 加班上班
        if(downpm && topot){
            Verify.notTrue(downTimePm.isAfter(topTimeOt), Verify.Attend.PmDownOtTop);
        }
        //三组时间至少设置一组
        Verify.isTrue(am || pm || ot, Verify.Attend.missingTime);

        MaintenanceCompany entity = maintenanceCompanyService.getById(request.getMtCompanyId());
        Verify.notNull(entity);

        entity.setTopTime(topTime);
        entity.setDownTime(downTime);
        entity.setTopTimePm(topTimePm);
        entity.setDownTimePm(downTimePm);
        entity.setTopTimeOt(topTimeOt);
        entity.setDownTimeOt(downTimeOt);

        boolean result = maintenanceCompanyService.saveOrUpdate(entity);
        return RestResponse.success(result);
    }

    /**
     * 打卡
     * 11 上午上班 12 上午下班
     * 21 下午上班 22 下午下班
     * 31 加班上班 32 加班下班
     *
     * @param entity Attendance
     * @return RestResponse
     */
    @PostMapping("clock")
    public RestResponse clock(@Valid @RequestBody Attendance entity) {
        Long mtCompanyId = entity.getMtCompanyId();
        Long userId = entity.getUserId();
        Integer type = entity.getType();
        //先统计当天是否有对应的打卡记录
        int count = attendanceService.countByUserAndType(mtCompanyId,userId,type);
        Verify.nogt0(count,Verify.Attend.hadClock);
        MaintenanceCompany maintenanceCompany = maintenanceCompanyService.getById(mtCompanyId);
        Verify.notNull(maintenanceCompany);

        LocalTime now = LocalTime.now();
        //状态（是否迟到早退，0：否，1：是）
        switch (type) {
            case 11:
                LocalTime topTime = maintenanceCompany.getTopTime();
                Verify.notNull(topTime, Verify.Attend.SetAmTime);
                //打卡时间比上班时间早-正常，否则迟到
                entity.setStatus(now.isBefore(topTime) ? 0 : 1);
                entity.setRemarks(Verify.Attend.AmTopClock);
                break;
            case 12:
                LocalTime downTime = maintenanceCompany.getDownTime();
                Verify.notNull(downTime, Verify.Attend.SetAmTime);
                //打卡时间比下班时间晚-正常，否则早退
                entity.setStatus(now.isAfter(downTime) ? 0 : 1);
                entity.setRemarks(Verify.Attend.AmDownClock);
                break;
            case 21:
                LocalTime topTimePm = maintenanceCompany.getTopTimePm();
                Verify.notNull(topTimePm, Verify.Attend.SetPmTime);
                entity.setStatus(now.isBefore(topTimePm) ? 0 : 1);
                entity.setRemarks(Verify.Attend.PmTopClock);
                break;
            case 22:
                LocalTime downTimePm = maintenanceCompany.getDownTimePm();
                Verify.notNull(downTimePm, Verify.Attend.SetPmTime);
                entity.setStatus(now.isAfter(downTimePm) ? 0 : 1);
                entity.setRemarks(Verify.Attend.PmDownClock);
                break;
            case 31:
                LocalTime topTimeOt = maintenanceCompany.getTopTimeOt();
                Verify.notNull(topTimeOt, Verify.Attend.SetOtTime);
                entity.setStatus(now.isBefore(topTimeOt) ? 0 : 1);
                entity.setRemarks(Verify.Attend.OtTopClock);
                break;
            case 32:
                LocalTime downTimeOt = maintenanceCompany.getDownTimeOt();
                Verify.notNull(downTimeOt, Verify.Attend.SetOtTime);
                entity.setStatus(now.isAfter(downTimeOt) ? 0 : 1);
                entity.setRemarks(Verify.Attend.OtDownClock);
                break;
            default:
                return RestResponse.failParam();
        }
        boolean result = attendanceService.save(entity);
        return RestResponse.success(result);
    }

    /**
     * 返回维保公司打卡时间信息
     * @param request AttendanceRequest
     * @return RestResponse
     */
    @PostMapping("time")
    @Verifier(fields = {"mtCompanyId"})
    public RestResponse time(@Ver @RequestBody AttendanceRequest request) {
        MaintenanceCompany entity = maintenanceCompanyService.getById(request.getMtCompanyId());
        return RestResponse.success(entity);
    }

    /**
     * 根据用户查询打卡记录
     * @param request AttendanceRequest
     * @return RestResponse
     */
    @PostMapping("pageByUser")
    @Verifier(fields = {"userId","clockTime"})
    public RestResponse pageByUser(@Ver @RequestBody AttendanceRequest request) {
        IPage<AttendanceResponse> pages = attendanceService.pageByUser(request);
        return RestResponse.success(pages);
    }

    @PostMapping("export")
    @Verifier(fields = {"ids","exportPath"})
    public void export(@Ver @RequestBody AttendanceRequest request, HttpServletResponse response){
        log.info("headerAlias: {}",headerAlias);
        List<AttendanceResponse> attendances = attendanceService.listByIdList(request);
        Verify.notNull(attendances);
        try {
            ExcelWriter writer = ExcelUtil.getWriter(request.getExportPath());
            //设置列别名
            headerAlias.forEach(writer::addHeaderAlias);
            //只导出配置了别名的列属性
            writer.setOnlyAlias(true);
            writer.write(attendances, true);
            writer.autoSizeColumnAll();
            response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet;charset=utf-8");
            response.setHeader("Content-Disposition","attachment;filename=" + IdWorker.getIdStr() + ".xlsx");
            ServletOutputStream out = response.getOutputStream();
            writer.flush(out, true);
            writer.close();
            IoUtil.close(out);
        }catch (Exception e){
            log.error("导出打卡记录异常", e);
            throw Verify.verifyException("导出打卡记录异常");
        }
    }
}
